//  
//  Copyright (C) 2011 Avi Romanoff <aviromanoff@gmail.com>
// 
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
// 

// Sub-class the base plug from libpantheon.

const string WALLPAPER_DIR = "/usr/share/backgrounds";

public class FileOperator : GLib.Object {

    private static bool is_valid_file_type (string fname) {

        string fname_down = fname.down();
        return (fname_down.has_suffix(".png") || fname_down.has_suffix(".jpeg") || fname_down.has_suffix(".jpg") || fname_down.has_suffix(".gif"));
    }

    public static int count_wallpapers (GLib.File music_folder) {

        GLib.FileInfo file_info = null;
        int index = 0;
        try {
            var enumerator = music_folder.enumerate_children(FILE_ATTRIBUTE_STANDARD_NAME + "," + FILE_ATTRIBUTE_STANDARD_TYPE, 0);
            while ((file_info = enumerator.next_file ()) != null) {
                if (file_info.get_file_type() == GLib.FileType.REGULAR && is_valid_file_type(file_info.get_name())) {
                    index++;
                }
            }
        } catch(GLib.Error err) {
            stdout.printf("Could not pre-scan wallpaper folder. Progress percentage may be off: %s\n", err.message);
        }
        return index;
    }

}

public class WallpapersPlug : Pantheon.Switchboard.Plug {

    Gtk.ListStore store = new Gtk.ListStore (2, typeof (Gdk.Pixbuf), typeof (string));
    // Needed?
    Gtk.TreeIter selected_plug;

    public WallpapersPlug () {

        Wallpaper.WallpaperSettings wallpaper_settings = new Wallpaper.WallpaperSettings ();
        setup_ui();
        gather_wallpapers_async();
    }

    private void setup_ui () {

        var vbox = new Gtk.VBox(false, 0);
        var sw = new Gtk.ScrolledWindow(null, null);
        var vp = new Gtk.Viewport(null, null);
        var view = new Gtk.IconView.with_model(store);

        vp.set_shadow_type(Gtk.ShadowType.NONE);
        sw.set_policy (Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC);
        sw.set_shadow_type(Gtk.ShadowType.NONE);
        sw.border_width = 0;
        view.selection_changed.connect(() => selection_changed_cb(view));
        view.set_pixbuf_column (0);
        view.set_text_column (1);

        vp.add(view);
        sw.add(vp);
        vbox.pack_end(sw, true, true, 0);
        this.add(vbox);
    }

    private void selection_changed_cb(Gtk.IconView view) {

        var selected = view.get_selected_items ();
        if (selected.length() == 1) {
            GLib.Value filename;
            var item = selected.nth_data(0);
            this.store.get_iter(out this.selected_plug, item);
            this.store.get_value(this.selected_plug, 1, out filename);
            // UPDATE WALLPAPER
//            this.wallpaper_controller.update_wallpaper(WALLPAPER_DIR + "/" + filename.get_string(), 1);    
        }
    }

    private async void gather_wallpapers_async () {
        
        switchboard_controller.progress_bar_set_visible(true);
        switchboard_controller.progress_bar_set_text("Importing wallpapers from " + WALLPAPER_DIR);

        var directory = File.new_for_path (WALLPAPER_DIR);
        double done = 0.0;
        int count = FileOperator.count_wallpapers(directory);
        var e = yield directory.enumerate_children_async (FILE_ATTRIBUTE_STANDARD_NAME,
                                                    0, Priority.DEFAULT);
        
        while (true) {
            var files = yield e.next_files_async (10, Priority.DEFAULT);
            if (files == null) {
                break;
            }
            foreach (var info in files) {
                done++;
                string? filename = (string) info.get_name ();
                Gtk.TreeIter root;
                this.store.append(out root);
                try {
                    var image = new Gdk.Pixbuf.from_file_at_size(WALLPAPER_DIR + "/" + filename, 100, 100);
//                    var color = Wallpaper.Utilities.average_color(image);
//                    string color_name = Wallpaper.Utilities.match_color_hsv(color);
                    this.store.set(root, 0, image, -1);
                    this.store.set(root, 1, filename, -1);
                } catch {
                }
                switchboard_controller.progress_bar_set_fraction(done/count);
                // Is this needed?
                while(Gtk.events_pending ()) {
                    Gtk.main_iteration();
                }
            }
        }
        switchboard_controller.progress_bar_set_visible(false);
    }

}

public static int main (string[] args) {

    Gtk.init (ref args);
    // Instantiate the plug, which handles
    // connecting to Switchboard.
    var plug = new WallpapersPlug ();
    // Connect to Switchboard and identify
    // as "Wallpapers". (For debugging)
    plug.register ("Wallpapers");
    plug.show_all ();
    // Start the GTK+ main loop.
    Gtk.main ();
    return 0;
}
